// file:    safe_array.H
// author:  R. Keller
// purpose: create generic array of arbitrary type that remembers its length
//          and which can check for out of bounds, and also trace accesses
//          by means of user specifiable procedures

// To create a N element array A of type T
//    safe_array<T>A(N);.
// To access element i of A:
//    A[i]

#include <iostream>
#include <stdlib.h>
#include "array.H"

template <class T> 
class safe_array : public array<T>
{
public:

safe_array();                                 // constructors
safe_array(long Length);
safe_array(long Length,                       // length of the array
	      void (*Range_error)(long, long) // out of range procedure
             ); 

T& operator[](long i);                        // element access

safe_array<T> & operator=(safe_array<T> & orig);

~safe_array();                                // destructor

// public inherited from array class
int length();                               // get length

private:

void (*range_error)(long, long);              // error routine
};


// This procedure is called on a range error, unless the user defines another

void default_range_error(long i, long Elements)
{
std::cerr << "index " << i << " out of range 0:" << Elements-1 << std::endl;
exit(1);
}

template <class T>
safe_array<T>::safe_array()           // default constructor
  : array<T>(), range_error(default_range_error)
{}

template <class T>                          // constructor
safe_array<T>::
safe_array(long Length)
  : array<T>(Length), range_error(default_range_error)
{}

template <class T>                          // constructor
safe_array<T>::
safe_array(long Length,                     // length of the array
	      void (*Range_error)(long, long)
             ) 
  : array<T>(Length), range_error(Range_error)
{}

template <class T>
T& safe_array<T>::operator[](long i)        // element access
{
if( range_error && ((i < 0 ) || (i >= array<T>::Length)) )
  {
  (*range_error)(i, array<T>::Length);
  return array<T>::operator[](0);
  }
return array<T>::operator[](i);
}

template <class T>
safe_array<T> & safe_array<T>::operator=(safe_array<T> & orig)	// assignment
{
array<T>::operator=(orig);		// assign to underlying array
range_error = orig.range_error;
}

template <class T>
safe_array<T>::~safe_array()                    // destructor
{}
